==> jmb.tools-0.2.8.tar.gz: jmb.tools-0.2.8/jmb/tools/kick/kickstart.py <==
#!/usr/bin/python
# coding:utf-8
"""
Esempi:

  jmb-start -t prj www.thundersystems.it web
  jmb-start -t cms www.thundersystems.it web
  jmb-start -t cms www.thundersystems.it web  jmb.fax jmb.news
  jmb-start -t app jmb.fax

usage: %prog [opts] app_name/prj_name [python_module_name [app1 app2 ...]]
   -t, --type=TYPE: the type of the new project/application
   -l, --list: list available types
   -e, --force-exists: untar even if a dir with same name already exists
   -L, --local: use local pypi (http://localhost/pypi)
   -f, --skeleton-file=SKELETON_FILE: use SKELETON_FILE as skeleton (must be tar file)
   -v, --verbose: be verbose, at least a little...
   -B, --bootstrap: lnacia 'python bootstrap -d'
   -b, --buildout: lancia bin/buildout -N (implica -B e -m)
   -g, --go: lancia jmb-go
   -m, --mercurial: inizilizza un repo mercurial
   -D, --debug: show variable internal to jmb-start
   -x, --no-cms: disable cms
   -a, --all: -xbgmv
   -d, --prj-dir=DIR:  create the project within dir (eg: jmb-start -d /tmp -t cms...)
   -k, --key-generation: generate a key and exit
"""
import os
import io
import re
import sys
import glob
import urllib2
import shutil
import tarfile
import StringIO
import subprocess
import random

from jmb.tools.functions import execute

BASE_TEMPLATE_URL = 'http://pypi.thundersystems.it/skeletons/'
PACKAGE_NAME = 'jmb-default-pkg'
PYTHON_MODULE = 'jmb_python_module'
PROJECT_PATTERN = '^$|cms|prj|p15|16|17|18|19|110'
APP_LABEL = '\$app_label'  # nota il $!!!  vale anche $App_Lable => Fax
# Es jmb.fax $app_label viene sostituoto con fax/Fax

# Use TAR_NAMES just if you want to list with -l options
# or if you want to map to a different name
TAR_NAMES = {
    'cms': 'cms141',
    'mod': 'mod',
    'app': 'app',
    'japp': 'japp',
    'japp2': 'japp2',
    'p15': 'p15',
    'p16': 'p16',
    'p17': 'p17',
    'p18': 'p18',
    'p318': 'p318',
    'p19': 'p19',
    'p110': 'p110',
    # 'jprj': 'jmb2-default-prj',
    'default': 'jmb2-default-prj',
}

def get_boundle_file_obj(prj_type):
    """Get a boundle
    Return a fileobject suitable for tar
    """
    PRJ_TAR_NAME = TAR_NAMES.get(prj_type, prj_type) + '.tar.gz'

    URL = BASE_TEMPLATE_URL + PRJ_TAR_NAME
    if opts.skeleton_file:
        URL = 'file://%s' % os.path.abspath(opts.skeleton_file)
    if opts.verbose:
        print("SKELETON URL:", URL)
    return StringIO.StringIO(urllib2.urlopen(URL).read())

def untar_boundle(prj_type='default'):

    if os.path.exists(PACKAGE_NAME):
        shutil.rmtree(PACKAGE_NAME)
    tar = tarfile.open(fileobj=get_boundle_file_obj(prj_type))
    tar.extractall()

def create_package_namespace(start_dir, pkg_name):
    """Create a python setup to hold pkg_name

    :param pkg_name: a package name, can be any level (eg: jmb.fax or jmb.ecommerce.basket)
    """
    prev = start_dir
    parts = pkg_name.split('.')

    for pkg in parts:
        new_dir = os.path.join(prev, pkg)
        try:
            os.mkdir(new_dir)
        except OSError:
            pass
        prev = new_dir

        # All dirs but the last one need a declare_namespace
        f = open(os.path.join(new_dir, '__init__.py'), 'w')
        if not pkg == parts[-1]:
            f.write("__import__('pkg_resources').declare_namespace(__name__)" + '\n')
        f.close()
    fix_namespace_in_setup(start_dir, parts[:-1])
    return new_dir

def fix_namespace_in_setup(dirname, namespace):

    if not namespace:
        return
    txt = open(os.path.join(dirname, 'setup.py'), 'r').read().split('\n')
    namespace_declaration = "    namespace_packages = ['%s']," % ".".join(namespace)
    for j, line in enumerate(txt):
        if re.search('\sname=.*', line):
            idx = j+1
    txt.insert(idx, namespace_declaration)
    f = open(os.path.join(dirname, 'setup.py'), 'w')
    f.write('\n'.join(txt))

def subst_vars(old_string, new_string, *file_pattern):
    """Substitute all occurencies of old_string with new string
    in file matching file_pattern

    :param old_string: the string to be substituted
    :param new_string: the new value for that string
    :param file_pattern: glob file pattern of file we want to operate on.
    """
    for pattern in file_pattern:
        pattern = "%s/%s/%s" % (os.getcwd(), PACKAGE_NAME, pattern)
        for filename in glob.glob(pattern):
            if os.path.isfile(filename):
                with io.open(filename, 'r', encoding="utf-8") as f:
                    content = f.read()
                    try:
                        content = re.sub(old_string, new_string, content)
                    except:
                        raise

                with io.open(filename, 'w', encoding="utf-8") as f:
                    f.write(content)

def copy_main_package():
    "Move the final product to real name. Check if dest exists"

    # templates/app_lable -> templates/fax
    template_dir = os.path.join(dest_module_dir, 'templates/app_label')

    if os.path.exists(template_dir):
        template_dir2 = 'templates/%s' % dest_module_dir.split('/')[-1]
        os.rename(template_dir, os.path.join(dest_module_dir, template_dir2))

    # static/app_lable -> templates/fax
    static_dir = os.path.join(dest_module_dir, 'static/app_label')

    if os.path.exists(static_dir):
        static_dir2 = 'static/%s' % dest_module_dir.split('/')[-1]
        os.rename(static_dir, os.path.join(dest_module_dir, static_dir2))

    if not os.path.exists(pkg_name):
        shutil.move(PACKAGE_NAME, pkg_name)
    else:
        if not opts.force_exists:
            print("Dir %s already exists, quitting" % pkg_name)
            sys.exit(1)
        else:
            dest_settings_dir = os.path.join(pkg_name, app_label, 'settings')
            preserve_local(dest_settings_dir, 'save')
            execute('rsync -a %s/ %s' % (PACKAGE_NAME, pkg_name))
            shutil.rmtree(PACKAGE_NAME)
            preserve_local(dest_settings_dir, 'restore')

    if os.path.exists('%s/hgignore-default' % pkg_name):
        os.rename('%s/hgignore-default' % pkg_name, '%s/.hgignore' % pkg_name)

def move_python_module(dest_module_dir):
    """Move python module to the final destination taking into account namespaces
    That means that jmb.core will be moved to jmb/core/
    """
    cmd = 'rsync -a {}/{}/ {}'.format(PACKAGE_NAME, PYTHON_MODULE, dest_module_dir)
    execute(cmd)
    shutil.rmtree(os.path.join(PACKAGE_NAME, PYTHON_MODULE))

def preserve_local(settings_dir, action):
    """
    Save and restore the local.py so that using jmb-start -e won't make you loose your data

    :arg settings:dir: the dir in the (possibly existing) destination
    :arg action: save or restore
    """

    filename = os.path.join(settings_dir, 'local.py')
    if action == 'save':
        if os.path.exists(filename):
            os.rename(filename, '/tmp/local.py')
    elif os.path.exists('/tmp/local.py'):
        os.rename('/tmp/local.py', filename)


def subst_placeholder_in_conf():
    "Substitute placeholder in conf file"

    subst_vars(PACKAGE_NAME, pkg_name, '*.py', '*.cfg', '*.in', 'docs/*py', 'docs/*rst',
               '%s/settings/*' % PYTHON_MODULE,
               )
    subst_vars(PYTHON_MODULE, python_module,
               '*.py',
               '*.cfg',
               '*.in',
               '.hgignore',
               'docs/*',
               '%s/*py' % PYTHON_MODULE,
               '%s/cms/*py' % PYTHON_MODULE,
               '%s/settings/*' % PYTHON_MODULE,
               '%s/tests/*' % PYTHON_MODULE,
               '%s/management/commands/*' % PYTHON_MODULE,
               )
    subst_vars(APP_LABEL, app_label,  # $app_label => fax
               '%s/*py' % PYTHON_MODULE,
               )
    subst_vars(APP_LABEL.title(), app_label.capitalize(),  # $App_Label => Fax
               '%s/*py' % PYTHON_MODULE,
               )
    subst_vars('temporary fake key', gen_secret_key(),  # a new secret key in local.py
               '%s/settings/local.py' % PYTHON_MODULE,
               )
    if opts.no_cms:
        subst_vars('CMS_ENABLED = True', 'CMS_ENABLED = False',
                   '%s/settings/*' % PYTHON_MODULE,
                   )
def setup_bo_hg(opts, pkg_name):
    """
    Setup buildout and mercurial if required by options
    """
    if (opts.mercurial or opts.buildout) and not os.path.exists(os.path.join(pkg_name, '.hg')):
        cmd = 'hg init '
        if opts.verbose:
            print(cmd)
        ret, out, err = execute(cmd, cwd=pkg_name)
        if ret:
            print(err)
        cmd = 'hg ci -A -m init'
        if opts.verbose:
            print(cmd)
        ret, out, err = execute(cmd, cwd=pkg_name)
        if ret:
            print(err)

    if re.search(PROJECT_PATTERN, opts.type or ''):
        ret, out, err = execute('ln -s dev.py settings.py',
                                cwd=os.path.join(pkg_name, python_module, 'settings'))
        if ret:
            print(err)

    if opts.bootstrap or opts.buildout:
        if opts.verbose:
            print('python bootstrap.py ', pkg_name)
        ret, out, err = execute('python bootstrap.py', cwd=pkg_name)
        if ret:
            print(err)
    if opts.buildout:
        if opts.verbose:
            print('bin/buildout -N')
        ret, out, err = execute('bin/buildout -N', cwd=pkg_name)
        if ret:
            print(err)

    if opts.go:
        if opts.verbose:
            print('jmb-go')
        ret, out, err = execute('jmb-go', cwd=pkg_name)
        if ret:
            print(err)

    # if opts.npm:
    #     if opts.verbose:
    #         print 'npm-fast-install install'
    #     ret, out, err = execute('npm-fast-install', cwd=pkg_name)
    #     if ret:
    #         print err


def show_variables():
    for varname in (
            'pkg_name',
            'python_module',
            'APP_LABEL',
            'app_label',
    ):
        print("%-15s: %s" % (varname, globals()[varname], ))


def setup_additional_packages():
    for pkg in additional_packages:

        subst_vars("(install_requires=\[[^\]]+)(    \])",
                   r"\1        '%s',\n\2" % pkg,
                   'setup.py'
                   )
        subst_vars("(INSTALLED_APPS = [^\)]+)",
                   r"\1    '%s',\n" % pkg,
                   '%s/settings/base.py' % PYTHON_MODULE,
                   )

def gen_secret_key():
    SECRET_KEY = ''.join([random.SystemRandom().choice(
        'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)])
    return SECRET_KEY


def main(opts):

    global dest_module_dir
    untar_boundle(opts.type or 'default')
    tmp_pkg_dir = os.path.join(os.getcwd(), PACKAGE_NAME)
    dest_module_dir = create_package_namespace(tmp_pkg_dir, python_module)
    subst_placeholder_in_conf()
    setup_additional_packages()
    move_python_module(dest_module_dir)
    copy_main_package()
    setup_bo_hg(opts, pkg_name)

def cli(options, args):
    global opts
    global python_module
    global pkg_name
    global app_label
    global additional_packages
    opts = options
    if opts.key_generation:
        output = "SECRET_KEY = '{}'\n".format(gen_secret_key())
        if len(args):
            with io.open(args[0], 'a') as f:
                f.write(output)
        else:
            print(output)
        sys.exit(0)

    if opts.prj_dir:
        os.chdir(opts.prj_dir)

    if opts.all:
        opts.go = True
        opts.buildout = True
        opts.bootstrap = True
        opts.mercurial = True
        opts.verbose = True
        opts.no_cms = True

    if opts.local:
        BASE_TEMPLATE_URL = 'http://localhost/pypi/'

    if opts.list:
        for key, value in TAR_NAMES.iteritems():
            print("   %-15.15s %s" % (key, value))
        sys.exit(0)

    if not args and not opts.list:
        subprocess.call(['jmb', 'kick', '--help'])
        return

    try:
        pkg_name = args[0].rstrip('/')
        python_module = args[1].rstrip('/')
    except IndexError:
        if re.search(PROJECT_PATTERN, opts.type or ''):
            python_module = 'web'
        else:
            python_module = pkg_name
    additional_packages = args[2:]
    app_label = python_module.split('.')[-1]

    if opts.debug:
        show_variables()

    main(opts)


####### Program
if __name__ == '__main__':
    pass
    # jmb-start -t japp jmb.fax
    #
    # ## really, this should no longer be used...
    # from dry import optionparse
    # opts, args = optionparse.parse(__doc__)

==> jmb.tools-0.4.tar.gz: jmb.tools-0.4/jmb/tools/kick/kickstart.py <==
#!/usr/bin/python
# coding:utf-8
"""
Esempi:

  jmb kick -t prj www.thundersystems.it web
  jmb kick -t cms www.thundersystems.it web
  jmb kick -t cms www.thundersystems.it web  jmb.fax jmb.news
  jmb kick -t app jmb.fax

usage: %prog [opts] app_name/prj_name [python_module_name [app1 app2 ...]]
   -t, --type=TYPE: the type of the new project/application
   -l, --list: list available types
   -e, --force-exists: untar even if a dir with same name already exists
   -L, --local: use local pypi (http://localhost/pypi)
   -f, --skeleton-file=SKELETON_FILE: use SKELETON_FILE as skeleton (must be tar file)
   -v, --verbose: be verbose, at least a little...
   -B, --bootstrap: lnacia 'python bootstrap -d'
   -b, --buildout: lancia bin/buildout -N (implica -B e -m)
   -g, --go: lancia jmb-go
   -m, --mercurial: inizilizza un repo mercurial
   -D, --debug: show variable internal to jmb kick
   -x, --no-cms: disable cms
   -a, --all: -xbgmv
   -d, --prj-dir=DIR:  create the project within dir (eg: jmb kick -d /tmp -t cms...)
   -k, --key-generation: generate a key and exit
"""
import os
import io
import re
import sys
import glob
import urllib2
import shutil
import tarfile
import StringIO
import subprocess
import random

from jmb.tools.functions import execute

BASE_TEMPLATE_URL = 'http://pypi.thundersystems.it/skeletons/'
PACKAGE_NAME = 'jmb-default-pkg'
PYTHON_MODULE = 'jmb_python_module'
PROJECT_PATTERN = '^$|cms|prj|p15|16|17|18|19|110|111'
APP_LABEL = '\$app_label'  # nota il $!!!  vale anche $App_Lable => Fax
# Es jmb.fax $app_label viene sostituoto con fax/Fax

# Use TAR_NAMES just if you want to list with -l options
# or if you want to map to a different name
TAR_NAMES = {
    'cms': 'cms141',
    'mod': 'mod',
    'app': 'app',
    'japp': 'japp',
    'japp2': 'japp2',
    'p15': 'p15',
    'p16': 'p16',
    'p17': 'p17',
    'p18': 'p18',
    'p318': 'p318',
    'p19': 'p19',
    'p110': 'p110',
    'p111': 'p111',
    # 'jprj': 'jmb2-default-prj',
    'default': 'jmb2-default-prj',
}

def get_boundle_file_obj(prj_type):
    """Get a boundle
    Return a fileobject suitable for tar
    """
    PRJ_TAR_NAME = TAR_NAMES.get(prj_type, prj_type) + '.tar.gz'

    URL = BASE_TEMPLATE_URL + PRJ_TAR_NAME
    if opts.skeleton_file:
        URL = 'file://%s' % os.path.abspath(opts.skeleton_file)
    if opts.verbose:
        print("SKELETON URL:", URL)
    return StringIO.StringIO(urllib2.urlopen(URL).read())

def untar_boundle(prj_type='default'):

    if os.path.exists(PACKAGE_NAME):
        shutil.rmtree(PACKAGE_NAME)
    tar = tarfile.open(fileobj=get_boundle_file_obj(prj_type))
    tar.extractall()

def create_package_namespace(start_dir, pkg_name):
    """Create a python setup to hold pkg_name

    :param pkg_name: a package name, can be any level (eg: jmb.fax or jmb.ecommerce.basket)
    """
    prev = start_dir
    parts = pkg_name.split('.')

    for pkg in parts:
        new_dir = os.path.join(prev, pkg)
        try:
            os.mkdir(new_dir)
        except OSError:
            pass
        prev = new_dir

        # All dirs but the last one need a declare_namespace
        f = open(os.path.join(new_dir, '__init__.py'), 'w')
        if not pkg == parts[-1]:
            f.write("__import__('pkg_resources').declare_namespace(__name__)" + '\n')
        f.close()
    fix_namespace_in_setup(start_dir, parts[:-1])
    return new_dir

def fix_namespace_in_setup(dirname, namespace):

    if not namespace:
        return
    txt = open(os.path.join(dirname, 'setup.py'), 'r').read().split('\n')
    namespace_declaration = "    namespace_packages = ['%s']," % ".".join(namespace)
    for j, line in enumerate(txt):
        if re.search('\sname=.*', line):
            idx = j+1
    txt.insert(idx, namespace_declaration)
    f = open(os.path.join(dirname, 'setup.py'), 'w')
    f.write('\n'.join(txt))

def subst_vars(old_string, new_string, *file_pattern):
    """Substitute all occurencies of old_string with new string
    in file matching file_pattern

    :param old_string: the string to be substituted
    :param new_string: the new value for that string
    :param file_pattern: glob file pattern of file we want to operate on.
    """
    for pattern in file_pattern:
        pattern = "%s/%s/%s" % (os.getcwd(), PACKAGE_NAME, pattern)
        for filename in glob.glob(pattern):
            if os.path.isfile(filename):
                with io.open(filename, 'r', encoding="utf-8") as f:
                    content = f.read()
                    try:
                        content = re.sub(old_string, new_string, content)
                    except:
                        raise

                with io.open(filename, 'w', encoding="utf-8") as f:
                    f.write(content)

def copy_main_package():
    "Move the final product to real name. Check if dest exists"

    # templates/app_lable -> templates/fax
    template_dir = os.path.join(dest_module_dir, 'templates/app_label')

    if os.path.exists(template_dir):
        template_dir2 = 'templates/%s' % dest_module_dir.split('/')[-1]
        os.rename(template_dir, os.path.join(dest_module_dir, template_dir2))

    # static/app_lable -> templates/fax
    static_dir = os.path.join(dest_module_dir, 'static/app_label')

    if os.path.exists(static_dir):
        static_dir2 = 'static/%s' % dest_module_dir.split('/')[-1]
        os.rename(static_dir, os.path.join(dest_module_dir, static_dir2))

    if not os.path.exists(pkg_name):
        shutil.move(PACKAGE_NAME, pkg_name)
    else:
        if not opts.force_exists:
            print("Dir %s already exists, quitting" % pkg_name)
            sys.exit(1)
        else:
            dest_settings_dir = os.path.join(pkg_name, app_label, 'settings')
            preserve_local(dest_settings_dir, 'save')
            execute('rsync -a %s/ %s' % (PACKAGE_NAME, pkg_name))
            shutil.rmtree(PACKAGE_NAME)
            preserve_local(dest_settings_dir, 'restore')

    if os.path.exists('%s/hgignore-default' % pkg_name):
        os.rename('%s/hgignore-default' % pkg_name, '%s/.hgignore' % pkg_name)

def move_python_module(dest_module_dir):
    """Move python module to the final destination taking into account namespaces
    That means that jmb.core will be moved to jmb/core/
    """
    cmd = 'rsync -a {}/{}/ {}'.format(PACKAGE_NAME, PYTHON_MODULE, dest_module_dir)
    execute(cmd)
    shutil.rmtree(os.path.join(PACKAGE_NAME, PYTHON_MODULE))

def preserve_local(settings_dir, action):
    """
    Save and restore the local.py so that using 'jmb kick' -e won't make you loose your data

    :arg settings:dir: the dir in the (possibly existing) destination
    :arg action: save or restore
    """

    filename = os.path.join(settings_dir, 'local.py')
    if action == 'save':
        if os.path.exists(filename):
            os.rename(filename, '/tmp/local.py')
    elif os.path.exists('/tmp/local.py'):
        os.rename('/tmp/local.py', filename)


def subst_placeholder_in_conf():
    "Substitute placeholder in conf file"

    subst_vars(PACKAGE_NAME, pkg_name, '*.py', '*.cfg', '*.in', 'docs/*py', 'docs/*rst',
               '%s/settings/*' % PYTHON_MODULE,
               )
    subst_vars(PYTHON_MODULE, python_module,
               '*.py',
               '*.cfg',
               '*.in',
               '.hgignore',
               'docs/*',
               '%s/*py' % PYTHON_MODULE,
               '%s/cms/*py' % PYTHON_MODULE,
               '%s/settings/*' % PYTHON_MODULE,
               '%s/tests/*' % PYTHON_MODULE,
               '%s/management/commands/*' % PYTHON_MODULE,
               )
    subst_vars(APP_LABEL, app_label,  # $app_label => fax
               '%s/*py' % PYTHON_MODULE,
               )
    subst_vars(APP_LABEL.title(), app_label.capitalize(),  # $App_Label => Fax
               '%s/*py' % PYTHON_MODULE,
               )
    subst_vars('temporary fake key', gen_secret_key(),  # a new secret key in local.py
               '%s/settings/local.py' % PYTHON_MODULE,
               )
    if opts.no_cms:
        subst_vars('CMS_ENABLED = True', 'CMS_ENABLED = False',
                   '%s/settings/*' % PYTHON_MODULE,
                   )
def setup_bo_hg(opts, pkg_name):
    """
    Setup buildout and mercurial if required by options
    """
    if (opts.mercurial or opts.buildout) and not os.path.exists(os.path.join(pkg_name, '.hg')):
        cmd = 'hg init '
        if opts.verbose:
            print(cmd)
        ret, out, err = execute(cmd, cwd=pkg_name)
        if ret:
            print(err)
        cmd = 'hg ci -A -m init'
        if opts.verbose:
            print(cmd)
        ret, out, err = execute(cmd, cwd=pkg_name)
        if ret:
            print(err)

    if re.search(PROJECT_PATTERN, opts.type or ''):
        ret, out, err = execute('ln -s dev.py settings.py',
                                cwd=os.path.join(pkg_name, python_module, 'settings'))
        if ret:
            print(err)

    if opts.bootstrap or opts.buildout:
        if opts.verbose:
            print('python bootstrap.py ', pkg_name)
        ret, out, err = execute('python bootstrap.py', cwd=pkg_name)
        if ret:
            print(err)
    if opts.buildout:
        if opts.verbose:
            print('bin/buildout -N')
        ret, out, err = execute('bin/buildout -N', cwd=pkg_name)
        if ret:
            print(err)

    if opts.go:
        if opts.verbose:
            print('jmb-go')
        ret, out, err = execute('jmb-go', cwd=pkg_name)
        if ret:
            print(err)

    # if opts.npm:
    #     if opts.verbose:
    #         print 'npm-fast-install install'
    #     ret, out, err = execute('npm-fast-install', cwd=pkg_name)
    #     if ret:
    #         print err


def show_variables():
    for varname in (
            'pkg_name',
            'python_module',
            'APP_LABEL',
            'app_label',
    ):
        print("%-15s: %s" % (varname, globals()[varname], ))


def setup_additional_packages():
    for pkg in additional_packages:

        subst_vars("(install_requires=\[[^\]]+)(    \])",
                   r"\1        '%s',\n\2" % pkg,
                   'setup.py'
                   )
        subst_vars("(INSTALLED_APPS = [^\)]+)",
                   r"\1    '%s',\n" % pkg,
                   '%s/settings/base.py' % PYTHON_MODULE,
                   )

def gen_secret_key():
    SECRET_KEY = ''.join([random.SystemRandom().choice(
        'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)])
    return SECRET_KEY


def main(opts):

    global dest_module_dir
    untar_boundle(opts.type or 'default')
    tmp_pkg_dir = os.path.join(os.getcwd(), PACKAGE_NAME)
    dest_module_dir = create_package_namespace(tmp_pkg_dir, python_module)
    subst_placeholder_in_conf()
    setup_additional_packages()
    move_python_module(dest_module_dir)
    copy_main_package()
    setup_bo_hg(opts, pkg_name)

def cli(options, args):
    global opts
    global python_module
    global pkg_name
    global app_label
    global additional_packages
    opts = options
    if opts.key_generation:
        output = u"SECRET_KEY = '{}'\n".format(gen_secret_key())
        ## FIXME... opts.type ugly but working
        if opts.type:
            with io.open(opts.type, 'a') as f:
                f.write(output)
        else:
            print(output)
        sys.exit(0)

    if opts.prj_dir:
        os.chdir(opts.prj_dir)

    if opts.all:
        opts.go = True
        opts.buildout = True
        opts.bootstrap = True
        opts.mercurial = True
        opts.verbose = True
        opts.no_cms = True

    if opts.local:
        BASE_TEMPLATE_URL = 'http://localhost/pypi/'

    if opts.list:
        for key, value in TAR_NAMES.iteritems():
            print("   %-15.15s %s" % (key, value))
        sys.exit(0)

    if not args and not opts.list:
        subprocess.call(['jmb', 'kick', '--help'])
        return

    try:
        pkg_name = args[0].rstrip('/')
        python_module = args[1].rstrip('/')
    except IndexError:
        if re.search(PROJECT_PATTERN, opts.type or ''):
            python_module = 'web'
        else:
            python_module = pkg_name
    additional_packages = args[2:]
    app_label = python_module.split('.')[-1]

    if opts.debug:
        show_variables()

    main(opts)


####### Program
if __name__ == '__main__':
    pass
    # ## really, this should no longer be used...
    # from dry import optionparse
    # opts, args = optionparse.parse(__doc__)
